<?php

/**
 * Řídící třída s konfigurací úložišť souborů s uživateli a místnostmi.
 */
class CoreManager implements ICoreManager, ILogable {

    public $PATH_TO_CURRENT_USERS = './internal/users.txt';
    //define('PATH_TO_INCOMING_USERS,','./external/users.txt');
    public $PATH_TO_INCOMING_USERS = './external/users.txt';

    public $PATH_TO_INCOMING_ROOMS = './external/rooms.txt';
    public $PATH_TO_CURRENT_ROOMS = './internal/rooms.txt';

    private $PATH_TO_INCOMING_ROOM = './external/';
    private $PATH_TO_CURRENT_ROOM = './internal/';

    private $fileLoader;
    private $diffEngine;
    private $acApi;

    public function __construct() {
        $this->fileLoader = new FileLoader();
        $this->diffEngine = new DiffEngine();
        $this->acApi = new AcApi();
    }

    /**
     * Provede update uživatelů:
     * 1) nově příchozí vytvoří
     * 2) stávající uživatelé, kteří chybějí v příchozích se smažou
     *
     */
    public function updateUsers() {
        
        $methodName = 'updateUsers';
                $this->logDebug('Entering method '.$methodName, $methodName);

        $currentUsers = $this->fileLoader->loadUsers($this->PATH_TO_CURRENT_USERS);
        $incomingUsers = null;
        $incomingUsers = $this->fileLoader->loadUsers($this->PATH_TO_INCOMING_USERS);
        
        $usersToCreate = $this->diffEngine->findNewUsers($currentUsers, $incomingUsers);

        foreach ($usersToCreate as $u) {
            $principalId = $this->acApi->createUser($u);
            if ($principalId < 0) {
                $this->logError('principalId: ['.$principalId.']', $methodName);
            }
        }

        $usersToDelete = $this->diffEngine->findUsersToDelete($currentUsers, $incomingUsers);
        foreach ($usersToDelete as $user) {
            $isOk = $this->acApi->deleteUser($user);
            if (! $isOk) {
                $this->logError('Failed to delete user: ['.$user->toString().']', $methodName);
            }
        }
    }

        /**
     * Provede update místností:
     * 1) nově příchozí vytvoří
     * 2) stávající místnosti, které chybějí v příchozích se smažou
     *
     */
    public function updateRooms() {

        $methodName = 'updateRooms';
                $this->logDebug('Entering method '.$methodName, $methodName);

        $currentRooms = $this->fileLoader->loadRooms($this->PATH_TO_CURRENT_ROOMS);
        $incomingRooms = null;
        $incomingRooms = $this->fileLoader->loadRooms($this->PATH_TO_INCOMING_ROOMS);
        
        $roomsToCreate = $this->diffEngine->findNewRooms($currentRooms, $incomingRooms);
        //ToDo createRoom...
        foreach ($roomsToCreate as $room) {
            $scoId = $this->acApi->createRoom($room);
            $this->logDebug('created room: ['.$room->toString().'] with scoId: ['.$scoId.']', $methodName);
            if ($scoId < 0 && $scoId != -10) { //-10 means already exists, thats no error
                $this->logError('Failed to create room: ['.$room->toString().'] with scoId: ['.$scoId.']', $methodName);
            }
        }
        $roomsToDelete = $this->diffEngine->findRoomsToDelete($currentRooms, $incomingRooms);
        foreach($roomsToDelete as $room) {
            $isOk = $this->acApi->deleteRoom($room);
            if (! isOk ) {
                $this->logError('Failed to delete room: ['.$room->toString().']', $methodName);
            }
        }
    }

        /**
     * Provede update uživatelů v jednotlivých místnostech, které jsou definovány
         * v souboru daném PATH_TO_INCOMING_ROOMS, takto:
     * 1) nově příchozím přidá právo
     * 2) stávající uživatelé, kterří chybějí v příchozích se právo odebere
     *
     */
    public function updateUsersOfRooms() {

        $methodName = 'updateUsersOfRooms';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $rooms = $this->fileLoader->loadRooms($this->PATH_TO_INCOMING_ROOMS);
        $roomAlreadyExists = true;
        foreach($rooms as $room) {
            try {
                //ToDo refactor: use check method isFileAccessible from FileLoader !!
                $currentUsers = $this->fileLoader->loadUsersOfRoom($this->PATH_TO_CURRENT_ROOM . trim($room->getName()).'.txt');
            } catch (Exception $e) {
                $this->logDebug('Either file of room ['.$room->toString().'] does not exists in ./internal or reading failed', $methodName);
                $roomAlreadyExists = false;
            }
            if ($roomAlreadyExists) {
                //udelej diff a updatuj zmeny
                $incomingUsers = $this->getIncomingUsersForRoom($room);
                $this->updateUsersInRoom($room, $currentUsers, $incomingUsers);
            } else {
                //create new room and add its users to it
                $scoId = $this->acApi->createRoom($room) ;
                if ($scoId < 0) {
                    $this->logError('Failed to create room ['.$room->toString().']', $methodName);
                    break; //try next room
                } else {
                    $incomingUsers = $this->getIncomingUsersForRoom($room);
                    $this->addUsersToRoom($room, $incomingUsers);
                }
            }
        }

    }

    /*
     * Provede update souboru
     * 1) smaze soucasne soubory
     * 2) zkopiruje soubory z prichozi slozky do slozky se soucasnymi soubory
     * 3) smaze soubory z prichozi slozky
     * Metoda nehlida existenci souboru, takze je treba zachazet opatrne aby
     * nedoslo k smazani vsech souboru (pokud napr. prichozi slozka bude prazdna,
     *  pak se smazou soucasne soubory a nebude ani co kopirovat)
     */
    public function updateFiles() {
        //smaz puvodni soubory a presun na jejich misto nove
        //$this->fileLoader->deleteFile($this->PATH_TO_CURRENT_USERS);
        //$this->fileLoader->moveFile($this->PATH_TO_INCOMING_USERS, $this->PATH_TO_CURRENT_USERS);
        //$this->fileLoader->deleteFile($this->PATH_TO_CURRENT_ROOMS);
        //$this->fileLoader->moveFile($this->PATH_TO_INCOMING_ROOMS, $this->PATH_TO_CURRENT_ROOMS);
        //smazeme vse ze stare slozky
        $this->fileLoader->deleteAllFiles($this->PATH_TO_CURRENT_ROOM);
        //a zkopirujeme obsah puvodni..
        $this->fileLoader->recurse_copy($this->PATH_TO_INCOMING_ROOM, $this->PATH_TO_CURRENT_ROOM);
// a smazeme obsah incoming/external..
        $this->fileLoader->deleteAllFiles($this->PATH_TO_INCOMING_ROOM);
    }

    // private functions -----------------------------------------------------


        /**
         *
         * Přidá uživatelé do místnosti room (nastaví jim práva).
         * @param <type> $room instance Room
         * @param <type> $users pole instancí User
         */
    private function addUsersToRoom($room, $users) {

        $methodName = 'addUsersToRoom';
        $this->logDebug('Entering method '.$methodName, $methodName);

        foreach ($users as $user) {
            //addToRoom
            $this->logDebug('Adding user ['.$user->toString().'] to room: ['.$room->toString().']', $methodName);
            $isOk = $this->acApi->updateUserToRoom($user, $room, Permission::HOST);
            if (! $isOk) { //false return, error occured
                //$this->processError('Adding user '.$user->toString().' to room '.$room->toString().' failed', 'addUsersToRoom');
                $this->logError('Adding user to room failed. Params: room: ['.$room->toString().'] user: ['.$user->toString().']', $methodName);
            }
        }
    }

    /**
     * Odebere uživatelům právo na přístup do místnosti room.
     * @param <type> $room instance Room
     * @param <type> $users pole instancí User
     */
    private function removeUsersFromRoom($room, $users) {

        $methodName = 'removeUsersFromRoom';
        $this->logDebug('Entering method '.$methodName, $methodName);

        foreach ($users as $user) {
            //remove from Room
           $this->logDebug('Removing user ['.$user->toString().'] from room: ['.$room->toString().']', $methodName);
            $isOk = $this->acApi->updateUserToRoom($user, $room, Permission::REMOVE);
            if (!$isOk) {
                $this->logError('Removing user from room failed. Params: room: ['.$room->toString().'] user: ['.$user->toString().']', $methodName);
            }
        }
    }

    /**
     * Vrátí pole nově příchozích uživatelů (oproti současným).
     * @param <type> $room instance Room - místnost, pro kterou uživatele zjišťujeme
     * @return <type> pole instancí Room
     */
    private function getIncomingUsersForRoom($room) {

        $methodName = 'getIncomingUsersForRoom';
        $this->logDebug('Entering method '.$methodName.' with param room: ['.$room->toString().']', $methodName);

        return $this->fileLoader->loadUsersOfRoom($this->PATH_TO_INCOMING_ROOM . trim($room->getName()).'.txt');
    }

    /**
     * Provede update uživatelů v místnosti. Nově příchozím právo přidá,
     * dalším odebere.
     * @param <type> $room instance Room
     * @param <type> $currentUsers pole instancí User
     * @param <type> $incomingUsers pole instancí User
     */
    private function updateUsersInRoom($room, $currentUsers, $incomingUsers) {


        $methodName = 'updateUsersInRoom';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $usersToRemoveFromRoom = $this->diffEngine->findUsersToDelete($currentUsers, $incomingUsers);
        //remove users that are not anymore in room
        $this->removeUsersFromRoom($room, $usersToRemoveFromRoom);
        //add new users
        $usersToAddToRoom = $this->diffEngine->findNewUsers($currentUsers, $incomingUsers);
        $this->addUsersToRoom($room, $usersToAddToRoom);
    }


    public function logDebug($description, $methodName) {
        SysLogger::logDebug($description, 'CoreManager', $methodName);
    }

    public function logError($description, $methodName) {
        SysLogger::logError($description, 'CoreManager', $methodName);
    }
}
?>
